@@ -42,6 +42,8 @@ CDasherModel::CDasherModel(CEventHandler *pEventHandler, CSettingsStore *pSettin
 m_pLanguageModel(NULL), m_pcAlphabet(NULL), m_Rootmin(0), m_Rootmax(0), m_Rootmin_min(0),
 m_Rootmax_max(0), m_dAddProb(0.0), m_dMaxRate(0.0) {
 
+  m_iTargetOffset = 0;
+
   m_bGameMode = bGameMode;
 
 #ifdef JAPANESE
@@ -205,9 +207,15 @@ void CDasherModel::Make_root(CDasherNode *whichchild)
   m_Rootmax = m_Rootmin + (range * m_Root->Hbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
   m_Rootmin = m_Rootmin + (range * m_Root->Lbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
   
-  myint iTargetRange = m_iTargetMax - m_iTargetMin;
-  m_iTargetMax = m_iTargetMin + (iTargetRange * m_Root->Hbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
-  m_iTargetMin = m_iTargetMin + (iTargetRange * m_Root->Lbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
+  for(std::deque<SGotoItem>::iterator it(m_deGotoQueue.begin()); it != m_deGotoQueue.end(); ++it) {
+    myint r = it->iN2 - it->iN1;
+    it->iN2 = it->iN1 + (r * m_Root->Hbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
+    it->iN1 = it->iN1 + (r * m_Root->Lbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
+  }
+
+//   myint iTargetRange = m_iTargetMax - m_iTargetMin;
+//   m_iTargetMax = m_iTargetMin + (iTargetRange * m_Root->Hbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
+//   m_iTargetMin = m_iTargetMin + (iTargetRange * m_Root->Lbnd()) / (int)GetLongParameter(LP_NORMALIZATION);
 }
 
 void CDasherModel::ClearRootQueue() {
@@ -269,6 +277,9 @@ void CDasherModel::Reparent_root(int lower, int upper) {
     
     pNewRoot = m_Root->m_pNodeManager->RebuildParent(m_Root, iGenerations);
 
+    //    std::cout << pNewRoot->m_pNodeManager << std::endl;
+    //    pNewRoot->m_bWatchDelete = true;
+
     lower = m_Root->Lbnd();
     upper = m_Root->Hbnd();
 
@@ -287,7 +298,7 @@ void CDasherModel::Reparent_root(int lower, int upper) {
   /* Determine how zoomed in we are */
 
   myint iRootWidth = m_Rootmax - m_Rootmin;
-  myint iTargetWidth = m_iTargetMax - m_iTargetMin;
+  //  myint iTargetWidth = m_iTargetMax - m_iTargetMin;
   myint iWidth = upper - lower;
 //  double scalefactor=(m_Rootmax-m_Rootmin)/static_cast<double>(upper-lower);
 
@@ -296,8 +307,8 @@ void CDasherModel::Reparent_root(int lower, int upper) {
   m_Rootmax = m_Rootmax + (myint((GetLongParameter(LP_NORMALIZATION) - upper)) * iRootWidth / iWidth);
   m_Rootmin = m_Rootmin - (myint(lower) * iRootWidth / iWidth);
   
-  m_iTargetMax = m_iTargetMax + (myint((GetLongParameter(LP_NORMALIZATION) - upper)) * iTargetWidth / iWidth);
-  m_iTargetMin = m_iTargetMin - (myint(lower) * iTargetWidth / iWidth);
+//   m_iTargetMax = m_iTargetMax + (myint((GetLongParameter(LP_NORMALIZATION) - upper)) * iTargetWidth / iWidth);
+//   m_iTargetMin = m_iTargetMin - (myint(lower) * iTargetWidth / iWidth);
   
   
 }
@@ -305,28 +316,28 @@ void CDasherModel::Reparent_root(int lower, int upper) {
 /////////////////////////////////////////////////////////////////////////////
 
 CDasherNode *CDasherModel::Get_node_under_crosshair() {
-  if(GetBoolParameter(BP_DELAY_VIEW))
-    return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
-  else
-    return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin, m_Rootmax, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
+//   if(GetBoolParameter(BP_DELAY_VIEW))
+//     return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
+//   else
+    return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
 CDasherNode *CDasherModel::Get_node_under_mouse(myint Mousex, myint Mousey) {
- if(GetBoolParameter(BP_DELAY_VIEW))
-    return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, Mousex, Mousey);
-  else
-    return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin, m_Rootmax, Mousex, Mousey);
+//  if(GetBoolParameter(BP_DELAY_VIEW))
+//     return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, Mousex, Mousey);
+//   else
+  return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, Mousex, Mousey);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
 void CDasherModel::Get_string_under_mouse(const myint Mousex, const myint Mousey, vector <symbol >&str) {
-  if(GetBoolParameter(BP_DELAY_VIEW))
-    m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, Mousex, Mousey, str);
-  else
-    m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin, m_Rootmax, Mousex, Mousey, str);
+ //  if(GetBoolParameter(BP_DELAY_VIEW))
+//     m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, Mousex, Mousey, str);
+//   else
+    m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, Mousex, Mousey, str);
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -355,6 +366,8 @@ void CDasherModel::Start() {
 
 void CDasherModel::SetContext(std::string &sNewContext) {
   
+  m_deGotoQueue.clear();
+
 
   if(oldroots.size() > 0) {
     delete oldroots[0];
@@ -396,8 +409,10 @@ void CDasherModel::SetContext(std::string &sNewContext) {
   m_Rootmin = GetLongParameter(LP_MAX_Y) / 2 - iWidth / 2;
   m_Rootmax = GetLongParameter(LP_MAX_Y) / 2 + iWidth / 2;
 
-  m_iTargetMin = m_Rootmin;
-  m_iTargetMax = m_Rootmax;
+  m_iTargetOffset = 0;
+
+//   m_iTargetMin = m_Rootmin;
+//   m_iTargetMax = m_Rootmax;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -424,11 +439,12 @@ void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey,
   // Avoid Mousex=0, as this corresponds to infinite zoom
 
   // If Mousex is too large we risk overflow errors, so make limit it
-  // (this is a somewhat empirical limit - at some point we should
-  // probably do it a little more scientifically)
+  int iSteps = m_fr.Steps();
 
-  if(Mousex > 60000000)
-    Mousex = 60000000;
+  int iMaxX = (1 << 29) / iSteps;
+
+  if(Mousex > iMaxX)
+    Mousex = iMaxX;
 
   // Cache some results so we don't do a huge number of parameter lookups
 
@@ -443,10 +459,6 @@ void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey,
   // point under the cursor is at the cross-hair. This is where 
   // we want to be in iSteps updates
 
-  //  std::cout << iTargetMin << " " << iTargetMax << std::endl;
-
-  int iSteps = m_fr.Steps();
-
   DASHER_ASSERT(iSteps > 0);
 
   // iSteps is the number of update steps we need to get the point
@@ -693,8 +705,10 @@ void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax) {
     m_Rootmax = newRootmax;
     m_Rootmin = newRootmin;
 
-    m_iTargetMax = m_iTargetMax + 0.1 * (m_Rootmax - m_iTargetMax);
-    m_iTargetMin =  m_iTargetMin + 0.1 * (m_Rootmin - m_iTargetMin);
+    m_iTargetOffset = (m_iTargetOffset * 90) / 100;
+    
+    //     m_iTargetMax = m_iTargetMax + 0.1 * (m_Rootmax - m_iTargetMax);
+//     m_iTargetMin =  m_iTargetMin + 0.1 * (m_Rootmin - m_iTargetMin);
   }
   else {
     // TODO - force a new root to be chosen, so that we get better
@@ -737,7 +751,6 @@ void CDasherModel::HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode) {
 //
 
 bool CDasherModel::DeleteCharacters(CDasherNode *newnode, CDasherNode *oldnode, int* pNumDeleted) {
-
   // DJW cant see how either of these can ever be NULL
   DASHER_ASSERT_VALIDPTR_RW(newnode);
   DASHER_ASSERT_VALIDPTR_RW(oldnode);
@@ -931,7 +944,7 @@ void CDasherModel::Push_Node(CDasherNode *pNode) {
   }
 
   pNode->Alive(true);
-  
+
   pNode->m_pNodeManager->PopulateChildren(pNode);
   pNode->SetHasAllChildren(true);
 }
@@ -942,9 +955,9 @@ void CDasherModel::Recursive_Push_Node(CDasherNode *pNode, int iDepth) {
     return;
   }
 
-  if(pNode->Symbol() == GetControlSymbol()) {
-    return;
-  }
+ //  if(pNode->Symbol() == GetControlSymbol()) {
+//     return;
+//   }
 
   Push_Node(pNode);
 
@@ -979,10 +992,10 @@ bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay) {
 
   bool bReturnValue;
 
-  if(GetBoolParameter(BP_DELAY_VIEW))
-    bReturnValue = pView->Render(m_Root, m_iTargetMin, m_iTargetMax, vNodeList, vDeleteList, bRedrawDisplay, m_bGameMode);
-  else
-    bReturnValue = pView->Render(m_Root, m_Rootmin, m_Rootmax, vNodeList, vDeleteList, bRedrawDisplay, m_bGameMode);
+  //if(GetBoolParameter(BP_DELAY_VIEW))
+  //  bReturnValue = pView->Render(m_Root, m_iTargetMin, m_iTargetMax, vNodeList, vDeleteList, bRedrawDisplay, m_bGameMode);
+  //else
+    bReturnValue = pView->Render(m_Root, m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, vNodeList, vDeleteList, bRedrawDisplay, m_bGameMode);
 
   if(!GetBoolParameter(BP_OLD_STYLE_PUSH)) {
   for(std::vector<CDasherNode *>::iterator it(vNodeList.begin()); it != vNodeList.end(); ++it)
@@ -997,8 +1010,8 @@ bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay) {
 
 bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
 
-  if(m_deGotoQueue.size() > 0)
-    return false;
+//  if(m_deGotoQueue.size() > 0)
+ //   return false;
 
   CDasherNode *root(m_Root);
   CDasherNode::ChildMap & children = m_Root->Children();
@@ -1099,17 +1112,26 @@ void CDasherModel::ScheduleZoom(int dasherx, int dashery) {
 }
 
 void CDasherModel::Offset(int iOffset) {
-  m_Rootmin = m_iTargetMin + iOffset;
-  m_Rootmax = m_iTargetMax + iOffset;
+  m_Rootmin += iOffset;
+  m_Rootmax += iOffset;
+
+  m_iTargetOffset -= iOffset;
 } 
 
 void CDasherModel::MatchTarget(bool bReverse) {
-  if(bReverse) {
-    m_iTargetMin = m_Rootmin;
-    m_iTargetMax = m_Rootmax;
-  }
-  else {
-    m_Rootmin = m_iTargetMin;
-    m_Rootmax = m_iTargetMax;
-  }
+  // TODO: Does anything need to happen wrt bReverse here?
+
+  m_Rootmin += m_iTargetOffset;
+  m_Rootmax += m_iTargetOffset;
+
+  m_iTargetOffset = 0;
+
+//   if(bReverse) {
+//     m_iTargetMin = m_Rootmin;
+//     m_iTargetMax = m_Rootmax;
+//   }
+//   else {
+//     m_Rootmin = m_iTargetMin;
+//     m_Rootmax = m_iTargetMax;
+//   }
 }
